package org.fuseri.moduleexercise.answer;

import jakarta.transaction.Transactional;
import org.fuseri.model.dto.exercise.AnswerCreateDto;
import org.fuseri.model.dto.exercise.AnswerDto;
import org.fuseri.model.dto.exercise.AnswersCreateDto;
import org.fuseri.moduleexercise.question.Question;
import org.fuseri.moduleexercise.question.QuestionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * Represent facade for managing answers
 * Provide simplified interface for manipulating with answers
 */
@Service
@Transactional
public class AnswerFacade {
    private final AnswerService answerService;
    private final QuestionService questionService;
    private final AnswerMapper mapper;


    /**
     * Constructor for AnswerFacade
     *
     * @param answerService   the service responsible for handling answer-related logic
     * @param questionService the service responsible for handling question-related logic
     * @param mapper          the mapper responsible for converting between DTOs and entities
     */
    @Autowired
    public AnswerFacade(AnswerService answerService, QuestionService questionService, AnswerMapper mapper) {
        this.answerService = answerService;
        this.questionService = questionService;
        this.mapper = mapper;
    }

    /**
     * Create a new answer for the given question ID
     *
     * @param dto the AnswerCreateDto object containing information about the answer to create
     * @return an AnswerDto object representing the newly created answer
     */
    public List<AnswerDto> createMultiple(@RequestBody AnswersCreateDto dto) {
        List<Answer> createdAnswers = new ArrayList<>();
        for (var answerDto : dto.getAnswers()) {
            Question question;
            question = questionService.find(dto.getQuestionId());

            Answer answer = mapper.fromCreateDto(answerDto);
            answer.setQuestion(question);
            var createdAnswer = answerService.create(answer);
            question.getAnswers().add(answer);
            createdAnswers.add(createdAnswer);
        }

        return mapper.toDtoList(createdAnswers);
    }

    /**
     * Update an answer
     *
     * @param id  of answer to update
     * @param dto dto with updated answer information
     */
    public AnswerDto update(long id, AnswerCreateDto dto) {
        var updatedAnswer = mapper.fromCreateDto(dto);
        updatedAnswer.setId(id);
        answerService.update(updatedAnswer);

        Question question;
        question = questionService.find(dto.getQuestionId());

        var questionAnswers = question.getAnswers();
        questionAnswers.removeIf(a -> a.getId() == id);
        questionAnswers.add(updatedAnswer);
        question.setAnswers(questionAnswers);
        questionService.update(question);

        return mapper.toDto(updatedAnswer);
    }

    /**
     * Delete answer with the given id
     *
     * @param id of answer to delete
     */
    public void delete(long id) {
        var answer = answerService.find(id);

        Question question;
        question = questionService.find(answer.getQuestion().getId());

        var questionAnswers = question.getAnswers();
        questionAnswers.removeIf(a -> a.getId() == answer.getId());
        question.setAnswers(questionAnswers);

        answerService.delete(id);
    }
}
