package org.fuseri.moduleexercise.common;

import org.fuseri.model.dto.common.DomainObjectDto;
import org.fuseri.model.dto.common.Result;
import org.mapstruct.Mapping;
import org.mapstruct.Mappings;
import org.springframework.data.domain.Page;

import java.util.List;

/**
 * Base interface for MapStruct Mappers
 * It defines methods to convert DTOs to entities and vice versa
 *
 * @param <T> the type of the entity
 * @param <S> the type of the DTO
 */
public interface DomainMapper<T extends DomainObject, S extends DomainObjectDto> {

    /**
     * Convert DTO to its corresponding entity
     *
     * @param dto DTO to be converted
     * @return corresponding entity created from DTO
     */
    T fromDto(S dto);

    /**
     * Convert entity to its corresponding DTO
     *
     * @param entity entity to be converted
     * @return corresponding DTO created from entity
     */
    S toDto(T entity);

    /**
     * Convert List of entities to List of corresponding DTOs
     *
     * @param entities entities to be converted
     * @return corresponding list of DTO objects
     */
    List<S> toDtoList(List<T> entities);

    /**
     * Convert a {@link org.springframework.data.domain.Page} containing entities to a
     * {@link Result} object containing a list of their corresponding DTO objects, along with
     * pagination information
     *
     * @param source the Page of entities to be converted
     * @return a Result object containing a list of DTO objects and pagination information
     */
    @Mappings({
            @Mapping(target = "total", expression = "java(source.getTotalElements())"),
            @Mapping(target = "page", expression = "java(source.getNumber())"),
            @Mapping(target = "pageSize", expression = "java(source.getSize())"),
            @Mapping(target = "items", expression = "java(toDtoList(source.getContent()))")
    })
    Result<S> toResult(Page<T> source);
}
