package org.fuseri.modulelanguageschool.user;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Content;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import jakarta.persistence.EntityNotFoundException;
import org.fuseri.model.dto.common.Result;
import org.fuseri.model.dto.user.UserAddLanguageDto;
import org.fuseri.model.dto.user.UserCreateDto;
import org.fuseri.model.dto.user.UserDto;
import org.fuseri.model.dto.user.UserLoginDto;
import org.fuseri.modulelanguageschool.course.Course;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import jakarta.validation.Valid;
import jakarta.validation.constraints.PositiveOrZero;


import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("/users")
public class UserController {


    private final UserFacade facade;

    @Autowired
    public UserController(UserFacade facade) {
        this.facade = facade;

    }

    @Operation(summary = "Get a user by Id", description = "Returns a user with specified Id")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "User with the specified Id is retrieved Successfuly",
                    content = @Content(schema = @Schema(implementation = UserDto.class)
                    )),
            @ApiResponse(responseCode = "404", description = "User with the specified ID was not found.")
    })
    @GetMapping("/{id}")
    public ResponseEntity<UserDto> find(@PathVariable Long id) {
        try {
            return ResponseEntity.ok(facade.find(id));
        } catch (EntityNotFoundException e) {
            return ResponseEntity.notFound().build();
        }
    }


    @Operation(summary = "Create a User", description = "Creates a new User.")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "201", description = "User created successfully."),
            @ApiResponse(responseCode = "400", description = "Invalid input.")
    })
    @PostMapping
    public ResponseEntity<UserDto> create(@Valid @RequestBody UserCreateDto dto) {
        UserDto user = facade.create(dto);
        return ResponseEntity.status(HttpStatus.CREATED).body(user);
    }

    @Operation(summary = "Delete a User with specified ID", description = "Deletes a User with the specified ID.")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "204", description = "User with the specified ID deleted successfully."),
    })
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteUser(@PathVariable Long id) {
        facade.delete(id);
        return ResponseEntity.noContent().build();

    }

    @Operation(summary = "Update a User", description = "Updates a User with the specified ID.")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "User with the specified ID updated successfully."),
            @ApiResponse(responseCode = "400", description = "Invalid input."),
            @ApiResponse(responseCode = "404", description = "User with the specified ID was not found.")
    })
    @PutMapping("/update/{id}")
    public ResponseEntity<UserDto> update(@PositiveOrZero @PathVariable Long id, @Valid @RequestBody UserCreateDto dto) {

        try {
            return ResponseEntity.ok(facade.update(id, dto));
        } catch (EntityNotFoundException e) {
            return ResponseEntity.notFound().build();
        }
    }

    @Operation(summary = "Get Users in paginated format", description = "Returns Users in paginated format.")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Successfully retrieved paginated Users"),
            @ApiResponse(responseCode = "400", description = "Invalid page number supplied"),
    })
    @GetMapping("/all")
    public ResponseEntity<Result<UserDto>> findAll(@PositiveOrZero @RequestParam int page) {

        return ResponseEntity.ok(facade.findAll(page));
    }

    //TODO: add authentication
    @PostMapping("/login")
    public ResponseEntity<String> login(@RequestBody UserLoginDto dto) {
        return ResponseEntity.ok(String.format("User %s has spawned", dto.getUsername()));
    }

    //TODO: add authentication
    @PostMapping("/logout/{id}")
    public ResponseEntity<String > logout(@PathVariable Long id) {
        return ResponseEntity.ok("user has logged out");
    }


    // TODO do according to Course
    @GetMapping("/finished/{id}")
    public List<Course> getFinished(@PathVariable Long id) {
        return new ArrayList<>();
    }

    // TODO do according to Course

    @GetMapping("/enrolled/{id}")
    public List<Course> getEnrolled(@PathVariable Long id) {
        return new ArrayList<>();
    }

    @Operation(summary = "adds a language", description = "adds a new language and proficiency to user")
    @ApiResponses(value = {
            @ApiResponse(responseCode = "200", description = "Successfully added a language"),
            @ApiResponse(responseCode = "404", description = "User with given Id does not exist"),
            @ApiResponse(responseCode = "400", description = "Invalid input")
    })
    @PutMapping("/addLanguage/{id}")
    public ResponseEntity<UserDto> addLanguage(@PathVariable Long id, @Valid @RequestBody UserAddLanguageDto body) {
        try {
            return ResponseEntity.ok(facade.addLanguage(id, body));
        } catch (EntityNotFoundException e) {
            return ResponseEntity.notFound().build();
        }
    }

}
